using Adaptation.Eaf.Management.ConfigurationData.CellAutomation;
using Adaptation.Ifx.Eaf.EquipmentConnector.File.Configuration;
using Adaptation.Shared;
using Adaptation.Shared.Duplicator;
using Adaptation.Shared.Methods;
using Adaptation.Shared.Metrology;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;

namespace Adaptation.FileHandlers.OpenInsight;

public class FileRead : Shared.FileRead, IFileRead
{

    private int _LastIndex;
    private readonly string _IqsConnectionString;
    private readonly string _OpenInsightFilePattern;
    private readonly string _OpenInsightApiECDirectory;
    private readonly ReadOnlyCollection<ModelObjectParameterDefinition> _IQSCopyCollection;

    public FileRead(ISMTP smtp, Dictionary<string, string> fileParameter, string cellInstanceName, int? connectionCount, string cellInstanceConnectionName, FileConnectorConfiguration fileConnectorConfiguration, string equipmentTypeName, string parameterizedModelObjectDefinitionType, IList<ModelObjectParameterDefinition> modelObjectParameters, string equipmentDictionaryName, Dictionary<string, List<long>> dummyRuns, Dictionary<long, List<string>> staticRuns, bool useCyclicalForDescription, bool isEAFHosted) :
        base(new Description(), false, smtp, fileParameter, cellInstanceName, connectionCount, cellInstanceConnectionName, fileConnectorConfiguration, equipmentTypeName, parameterizedModelObjectDefinitionType, modelObjectParameters, equipmentDictionaryName, dummyRuns, staticRuns, useCyclicalForDescription, isEAFHosted: connectionCount is null)
    {
        _LastIndex = -1;
        _MinFileLength = 10;
        _Logistics = new(this);
        _NullData = string.Empty;
        if (_FileParameter is null)
            throw new Exception(cellInstanceConnectionName);
        if (_ModelObjectParameterDefinitions is null)
            throw new Exception(cellInstanceConnectionName);
        if (!_IsDuplicator)
            throw new Exception(cellInstanceConnectionName);
        _IQSCopyCollection = new(GetProperties(cellInstanceConnectionName, modelObjectParameters, "IQS.Copy."));
        _IqsConnectionString = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "IQS.ConnectionString");
        _OpenInsightFilePattern = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "OpenInsight.FilePattern");
        _OpenInsightApiECDirectory = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "OpenInsight.Api.EC.Directory");
    }

    void IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, Exception exception)
    {
        bool isErrorFile = exception is not null;
        if (!isErrorFile && !string.IsNullOrEmpty(_Logistics.ReportFullPath))
        {
            FileInfo fileInfo = new(_Logistics.ReportFullPath);
            if (fileInfo.Exists && fileInfo.LastWriteTime < fileInfo.CreationTime)
                File.SetLastWriteTime(_Logistics.ReportFullPath, fileInfo.CreationTime);
        }
        Move(extractResults);
    }

    void IFileRead.WaitForThread() => WaitForThread(thread: null, threadExceptions: null);

    string IFileRead.GetEventDescription()
    {
        string result = _Description.GetEventDescription();
        return result;
    }

    List<string> IFileRead.GetHeaderNames()
    {
        List<string> results = _Description.GetHeaderNames();
        return results;
    }

    string[] IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, string to, string from, string resolvedFileLocation, Exception exception)
    {
        string[] results = Move(extractResults, to, from, resolvedFileLocation, exception);
        return results;
    }

    JsonProperty[] IFileRead.GetDefault()
    {
        JsonProperty[] results = _Description.GetDefault(this, _Logistics);
        return results;
    }

    Dictionary<string, string> IFileRead.GetDisplayNamesJsonElement()
    {
        Dictionary<string, string> results = _Description.GetDisplayNamesJsonElement(this);
        return results;
    }

    List<IDescription> IFileRead.GetDescriptions(IFileRead fileRead, List<Test> tests, IProcessData processData)
    {
        List<IDescription> results = _Description.GetDescriptions(fileRead, _Logistics, tests, processData);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.GetExtractResult(string reportFullPath, string eventName)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        if (string.IsNullOrEmpty(eventName))
            throw new Exception();
        _ReportFullPath = reportFullPath;
        DateTime dateTime = DateTime.Now;
        results = GetExtractResult(reportFullPath, dateTime);
        if (results.Item3 is null)
            results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(results.Item1, Array.Empty<Test>(), JsonSerializer.Deserialize<JsonElement[]>("[]"), results.Item4);
        if (results.Item3.Length > 0 && _IsEAFHosted)
            WritePDSF(this, results.Item3);
        UpdateLastTicksDuration(DateTime.Now.Ticks - dateTime.Ticks);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.ReExtract()
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        List<string> headerNames = _Description.GetHeaderNames();
        Dictionary<string, string> keyValuePairs = _Description.GetDisplayNamesJsonElement(this);
        results = ReExtract(this, headerNames, keyValuePairs);
        return results;
    }

    internal static string GetLines(Logistics logistics, List<pcl.Description> descriptions)
    {
        StringBuilder results = new();
        char del = '\t';
        pcl.Description x = descriptions[0];
        _ = results.Append(x.UniqueId).Append(del).
            Append(logistics.MesEntity).Append(del).
            Append(x.Reactor).Append(del).
            Append(x.RDS).Append(del).
            Append(x.PSN).Append(del).
            Append(x.Layer).Append(del).
            Append(x.Zone).Append(del).
            Append(x.Wafer).Append(del).
            Append(x.Lot).Append(del).
            Append(x.Plan).Append(del).
            Append(x.Date).Append(del).
            Append(x.WaferSize).Append(del).
            Append(x.Ccomp).Append(del).
            Append(x.Area).Append(del).
            Append(x.CondType).Append(del).
            Append(x.Model).Append(del).
            Append(x.StartVoltage).Append(del).
            Append(x.StopVoltage).Append(del).
            Append(x.RampRate).Append(del).
            Append(x.GLimit.Replace(" µ", string.Empty)).Append(del).
            Append(x.SetupFile).Append(del).
            Append(x.SetupFile).Append(del).
            Append(x.Folder).Append(del).
            Append(x.Pattern).Append(del).
            Append(x.RhoMethod).Append(del).
            Append(x.NAvgMean).Append(del).
            Append(x.NAvgStdDev).Append(del).
            Append(x.NAvgRadialGradient).Append(del).
            Append(x.NslMean).Append(del).
            Append(x.NslStdDev).Append(del).
            Append(x.NslRadialGradient).Append(del).
            Append(x.VdMean).Append(del).
            Append(x.VdStdDev).Append(del).
            Append(x.VdRadialGradient).Append(del).
            Append(x.FlatZMean).Append(del).
            Append(x.FlatZStdDev).Append(del).
            Append(x.FlatZRadialGradient).Append(del).
            Append(x.RhoAvgMean).Append(del).
            Append(x.RhoAvgStdDev).Append(del).
            Append(x.RhoAvgRadialGradient).Append(del).
            Append(x.RhoslMean).Append(del).
            Append(x.RhoslStdDev).Append(del).
            Append(x.RhoslRadialGradient).Append(del).
            Append(x.PhaseMean).Append(del).
            Append(x.PhaseStdDev).Append(del).
            Append(x.PhaseRadialGradient).Append(del).
            Append(x.GradeMean).Append(del).
            Append(x.GradeStdDev).Append(del).
            Append(x.GradeRadialGradient).Append(del).
            Append(x.RsMean).Append(del).
            Append(x.RsStdDev).Append(del).
            Append(x.RsRadialGradient);
        foreach (pcl.Description description in descriptions)
            _ = results.Append(del).Append(description.Index + 1).
                        Append(del).Append(description.NAvg).
                        Append(del).Append(description.Nsl).
                        Append(del).Append(description.Vd).
                        Append(del).Append(description.FlatZ).
                        Append(del).Append(description.RhoAvg).
                        Append(del).Append(description.Rhosl).
                        Append(del).Append(description.Phase).
                        Append(del).Append(description.Grade);
        return results.ToString();
    }

    private void SaveOpenInsightFile(string reportFullPath, DateTime dateTime, string logistics, List<pcl.Description> descriptions, Test[] tests)
    {
        bool isDummyRun = false;
        List<(Shared.Properties.IScopeInfo, string)> collection = new();
        string duplicateDirectory = Path.Combine(_FileConnectorConfiguration.SourceFileLocation, _CellInstanceName);
        if (!Directory.Exists(duplicateDirectory))
            _ = Directory.CreateDirectory(duplicateDirectory);
        string successDirectory = _FileConnectorConfiguration.AlternateTargetFolder;
        if (!Directory.Exists(Path.Combine(duplicateDirectory, "1")))
        {
            string parentParent = GetParentParent(_FileConnectorConfiguration.SourceFileLocation);
            if (parentParent.Contains(_CellInstanceName))
                parentParent = Path.GetDirectoryName(parentParent);
            duplicateDirectory = Path.Combine(parentParent, "Data");
            if (!Directory.Exists(duplicateDirectory))
                _ = Directory.CreateDirectory(duplicateDirectory);
        }
        string duplicateFile = Path.Combine(duplicateDirectory, Path.GetFileName(reportFullPath));
        if (descriptions.Count != 0 && tests.Length != 0)
        {
            string lines = GetLines(_Logistics, descriptions);
            if (!string.IsNullOrEmpty(lines))
            {
                _LastIndex += 1;
                long? subGroupId;
                if (_LastIndex >= _IQSCopyCollection.Count)
                    _LastIndex = 0;
                long breakAfter = dateTime.AddSeconds(_BreakAfterSeconds).Ticks;
                ModelObjectParameterDefinition modelObjectParameterDefinition = _IQSCopyCollection[_LastIndex];
                long preWait = _FileConnectorConfiguration?.FileHandleWaitTime is null ? dateTime.AddMilliseconds(1234).Ticks : dateTime.AddMilliseconds(_FileConnectorConfiguration.FileHandleWaitTime.Value).Ticks;
                if (string.IsNullOrEmpty(descriptions[0].Reactor) || string.IsNullOrEmpty(descriptions[0].PSN))
                    subGroupId = null;
                else
                    (subGroupId, int? _, string _) = FromIQS.GetCommandText(_IqsConnectionString, _Logistics, descriptions[0], breakAfter, preWait);
                if (subGroupId is null)
                    collection.Add(new(new ScopeInfo(tests[0], _OpenInsightFilePattern), lines));
                else
                    collection.Add(new(new ScopeInfo(tests[0], $"{subGroupId.Value} {_OpenInsightFilePattern}"), lines));
                if (_StaticRuns.TryGetValue(_Logistics.Sequence, out List<string> values))
                {
                    if (values.Count != 1)
                        throw new Exception($"{nameof(_StaticRuns)} {values.Count} != 1 {_Logistics.Sequence}!");
                    values[0] = $"{values[0]}|{subGroupId}";
                }
                string weekOfYear = _Calendar.GetWeekOfYear(_Logistics.DateTimeFromSequence, CalendarWeekRule.FirstDay, DayOfWeek.Sunday).ToString("00");
                FromIQS.Save(_OpenInsightApiECDirectory, _Logistics, reportFullPath, logistics, descriptions.First(), lines, subGroupId, weekOfYear);
                try
                { FromIQS.SaveCopy(_FileConnectorConfiguration.SourceFileLocation, _IqsConnectionString, modelObjectParameterDefinition.Name, modelObjectParameterDefinition.Value.Split('|')); }
                catch (Exception exception)
                {
                    string subject = string.Concat("Exception:", _CellInstanceConnectionName);
                    string body = string.Concat(exception.Message, Environment.NewLine, Environment.NewLine, exception.StackTrace);
                    try
                    { _SMTP.SendHighPriorityEmailMessage(subject, body); }
                    catch (Exception) { }
                }
            }
        }
        if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
            WaitForFileConsumption(dateTime, descriptions, isDummyRun, successDirectory, duplicateDirectory, collection, duplicateFile);
    }

    private Tuple<string, Test[], JsonElement[], List<FileInfo>> GetExtractResult(string reportFullPath, DateTime dateTime)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        Tuple<string, string[], string[]> pdsf = ProcessDataStandardFormat.GetLogisticsColumnsAndBody(reportFullPath);
        _Logistics = new Logistics(reportFullPath, pdsf.Item1);
        SetFileParameterLotIDToLogisticsMID();
        JsonElement[] jsonElements = ProcessDataStandardFormat.GetArray(pdsf);
        List<pcl.Description> descriptions = pcl.ProcessData.GetDescriptions(jsonElements);
        Test[] tests = (from l in descriptions select (Test)l.Test).ToArray();
        if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
            SaveOpenInsightFile(reportFullPath, dateTime, pdsf.Item1, descriptions, tests);
        results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(pdsf.Item1, tests, jsonElements, new List<FileInfo>());
        return results;
    }

}